#!/usr/bin/perl -w

# $Id: operahotlist2html.pl,v 2.1 2003/02/19 05:26:06 alanh Exp alanh $
# Copyright (C) 2003 Alan Hoyle <alan@alanhoyle.com>
# 
# Converts Opera Hotlist files into HTML.
#
# This program is a derivitive of:  
#
# hotlist2xbel.pl
# Copyright (C) 2001 Haakon Nilsen <haakon@nilsen.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
#
# $Log: operahotlist2html.pl,v $
# Revision 2.1  2003/02/19 05:26:06  alanh
# New version
# added command line options.
# unified with myophotlist2html
#
# Revision 2.2  2003/02/19 05:11:55  alanh
# reindented whole program
# modified help message print statement
#
# Revision 2.1  2003/02/19 04:57:04  alanh
# added description to opening comments
#
# Revision 1.16  2003/02/18 22:27:53  alanh
# removed extra log file
#
# Revision 1.15  2003/02/18 22:26:37  alanh
# changed URI for valid CSS image.
# removed anybrowser image.
#
# Revision 1.14  2003/02/18 21:28:06  alanh
# Forgot a ');'  d'oh!
#
# Revision 1.13  2003/02/18 21:19:41  alanh
# added smart command line options instead of forced ones.
#
# Revision 1.12  2003/02/13 20:40:55  alanh
# added internal RCS log
#


use Time::Local;
use strict;
use Getopt::Long;

my $VERSION = '$Revision: 2.1 $';

my $defhr = '<hr>'; 

my $defheader ='<h1>Opera Hotlist</h1>';
my $deftitle =  'Opera Hotlist';


my $myheader = $defheader;
my $mytitle =  $deftitle;
my $mystyle = "";
my $myhr = $defhr;
my $HELP = 0;
my $infile = 0;
my $outfile = 0;

GetOptions ('t|title:s' => \$mytitle,
	    'd|header:s' => \$myheader,
            's|style:s' => \$mystyle,
	    'r|hr:s' => \$myhr,
	    'i|input:s' => \$infile,
	    'o|output:s' => \$outfile,
	    'h|help' => \$HELP);


my $INFILE = *STDIN;
my $OUTFILE = *STDOUT;
my $err = *STDERR;


if ($infile) { open ($INFILE, "< $infile") or die "can't open $infile: $!";}
if ($outfile) { open ($OUTFILE, "> $outfile") or die "can't open $outfile: $!";}


my @folders;

my $indent = 1;  

sub get_next_line {
    if (!eof($INFILE)) {
	$a = <$INFILE>;
	$a =~ s/\n//; # Chop didn't work...
	$a =~ s/\t//;
	return $a;
    }
}

sub write_html {
    print $OUTFILE (&get_indent ."$_[0]\n");
}

sub parse {
    &inc_indent;
    while (!eof($INFILE)) {
	my $line = get_next_line;
      SWITCH: {
	  if ($line eq "#FOLDER") {
	      &parse_folder;
	      last SWITCH;
	  }
	  if ($line eq "#URL") {
	      &parse_url;
	      last SWITCH;
	  }
      }
    }
    &dec_indent;
}

sub parse_folder {
    my ($line, $name, $value);
    my %attrib;
    $line = get_next_line;
    while ($line ne "") {
	if ($line =~ /=/) {
	    ($name, $value) = split(/=/, $line,2);
	    $attrib{$name} = $value;
	}
	$line = get_next_line;
    }
    
    $attrib{"NAME"} =~ s/\&amp\;/\&/g;
    $attrib{"NAME"} =~ s/\&gt\;/>/g;
    $attrib{"NAME"} =~ s/\&lt\;/</g;
    $attrib{"NAME"} =~ s/\&quot\;/\"/g;
    $attrib{"NAME"} =~ s/\&nbsp\;/ /g;
    
    $attrib{"NAME"} =~ s/\&/\&amp\;/g;
    $attrib{"NAME"} =~ s/\>/\&gt\;/g;
    $attrib{"NAME"} =~ s/\</\&lt\;/g;
    $attrib{"NAME"} =~ s/\"/\&quot\;/g;
    
    push @folders, $attrib{"NAME"};
    &write_html("<li><h3><a name=\"".$attrib{"NAME"} ."\"></a>" .$attrib{"NAME"} ."</h3>");
    
    &write_html("<ul>");
    &inc_indent;
    
    my $has_entries = 0;
    
    while ($line ne "-") {
	$line = get_next_line;
      SWITCH: {
	  if ($line eq "#FOLDER") {
	      &parse_folder;
	      $has_entries = 1;
	      last SWITCH;
	  }
	  if ($line eq "#URL") {
	      &parse_url;
	      $has_entries = 1;
	  }
      }
    }

    if ($has_entries == 0) 
    {
	&write_html ("<li>&nbsp;</li>");
    }

    &dec_indent;
    &write_html("</ul>");
}

sub parse_url {
    my (%bookmark, $line, $name, $value);
    $line = get_next_line;

    while ($line ne "") {
	if ($line =~ /=/) {
	    ($name, $value) = split(/=/, $line,2);
	    $bookmark{$name} = $value;
	}
	$line = get_next_line;
    }
    
    $bookmark{"NAME"} =~ s/\&amp\;/\&/g;
    $bookmark{"NAME"} =~ s/\&gt\;/\>/g;
    $bookmark{"NAME"} =~ s/\&lt\;/\</g;
    $bookmark{"NAME"} =~ s/\&quot\;/\"/g;
    $bookmark{"NAME"} =~ s/\&nbsp\;/ /g;
    
    $bookmark{"NAME"} =~ s/\&/\&amp\;/g;
    $bookmark{"NAME"} =~ s/\>/\&gt\;/g;
    $bookmark{"NAME"} =~ s/\</\&lt\;/g;
    $bookmark{"NAME"} =~ s/\"/\&quot\;/g;
    
    $bookmark{"URL"} =~ s/\&amp\;/\&/g;
    $bookmark{"URL"} =~ s/\&/\&amp\;/g;
    
    &write_html("<li><a href=\"" .$bookmark{"URL"} ."\">" .$bookmark{"NAME"} ."</a></li>");
}

sub inc_indent {
    $indent++;
}

sub dec_indent {
    $indent--;
}

sub get_indent {
    my $ind = "";
    for (my $i = 0; $i < $indent - 1; $i++) { $ind .= "   "; }
    return $ind;
}

#
#  Beginning of regular program execution
#


if ($HELP) {
    print("operahotlist2html.pl $VERSION by Alan Hoyle <alan\@alanhoyle.com>\n",
	  "   derived from hotlist2xbel.pl 1.0 by Haakon Nilsen <haakon\@nilsen.com>\n",
	  "   Reads Opera Hotlist 2.0 from stdin, writes HTML to stdout.\n",
	  "   add additional data after the command to personalize the output.\n",
	  "  Usage:\n",
	  "      operahotlist2html.pl [options]\n",
	  "  Options:\n",
	  "       -i, --input=PATH\n",
	  "                Instead of STDIN, use this path to source opera.adr\n",
	  "       -o, --output=PATH\n",
	  "                Instead of STDOUT, use this path to output html file\n",
	  "       -d, --header=STRING  \n",
	  "                title for the HTML page(default: '$defheader')\n",
	  "       -t, --title=STRING \n",
	  "                title in HTML <head<title></title> (default: '$deftitle')\n",
	  "       -s, --style=STRING \n",
	  "                stylesheet for output HTML\n",
	  "       -r, --hr=STRING\n",
	  "                HTML for a separator (default:  '$defhr')\n",
	  "       -h, --help\n",
	  "                print this help message\n"
	  );
    exit;
}

my $l = get_next_line;
if ($l ne "Opera Hotlist version 2.0") {
    print($err, "\nHotlist version not supported! Only 2.0 supported. No soup for you.\n");
    exit;
} else {
    &write_html("<?xml version=\"1.0\" encoding=\"utf-8\" ?>");
    &write_html("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/REC-html401/loose.dtd\">");
    &write_html("<html>\n");
    &inc_indent;
    &write_html("<head>");
    &write_html("<title>$mytitle</title>");
    
    if ($mystyle ne "") {
	&write_html("<link rel=\"StyleSheet\" href=\"$mystyle\" type=\"text/css\">");
    }

    &write_html("</head>");
    &write_html("<body>");
    &inc_indent;
    &write_html("$myheader");
    my $time = scalar localtime;
    &write_html("<p>Synced with Opera Hotlist at:  "  .$time .".</p>");
    &write_html("<p><a href=\"#Folder list\">Folder list</a></p>");
    &write_html("$myhr");
    &write_html("<ul>");
    
    &parse;
    
    &write_html("</ul>");
    &write_html("$myhr");
    &write_html("<h4 style=\"text-align:center\"><a name=\"Folder list\"></a>Folder list:</h4>");
    &write_html("<p style=\"text-align:justify\">");
    foreach my $folder (@folders){
	&write_html("[<a href=\"#$folder\">$folder</a>]");
    }
    &write_html("</p>");
    &write_html("$myhr");
    
    &write_html("<div style=\"float:right\">");
    &inc_indent;
    &write_html("<a href=\"http://validator.w3.org/check/referer\"><img");
    &write_html("style=\"border:0;width:88px;height:31px\"");
    &write_html("src=\"http://www.w3.org/Icons/valid-html401\" alt=\"Valid HTML 4.01!\"/></a>");

    if ($mystyle ne "") {
	&write_html("<a href=\"http://jigsaw.w3.org/css-validator/check/referer\"><img");
	&write_html("style=\"border:0;width:88px;height:31px\"");
	&write_html("src=\"http://jigsaw.w3.org/css-validator/images/vcss.png\"");
	&write_html("alt=\"Valid CSS!\"/></a>");
    }

    &dec_indent;
    &write_html("</div>");
    
    
    &write_html("<p>Synced with Opera Hotlist at:  "  .$time .".<br>");
    &write_html("Generated with <a href=\"http://ophotlist2html.sf.net\">OperaHotlist2html</a></p>");
    
    &dec_indent;
    &write_html("</body>");
    &dec_indent;
    &write_html("</html>");
}

exit;








